@@ -347,6 +347,10 @@ void uwsgi_setup_workers() {
 	// allocate signal table
         uwsgi.shared->signal_table = uwsgi_calloc_shared(sizeof(struct uwsgi_signal_entry) * 256 * (uwsgi.numproc + 1));
 
+#ifdef UWSGI_ROUTING
+	uwsgi_fixup_routes();
+#endif
+
 }
 
 pid_t uwsgi_daemonize2() {
