@@ -45,9 +45,13 @@ struct uwsgi_buffer *uwsgi_routing_translate(struct wsgi_request *wsgi_req, stru
 	char *pass1 = data;
 	size_t pass1_len = data_len;
 
+	if (ur->condition_ub[wsgi_req->async_id] && ur->ovn[wsgi_req->async_id] > 0) {
+		pass1 = uwsgi_regexp_apply_ovec(ur->condition_ub[wsgi_req->async_id]->buf, ur->condition_ub[wsgi_req->async_id]->pos, data, data_len, ur->ovector[wsgi_req->async_id], ur->ovn[wsgi_req->async_id]);
+		pass1_len = strlen(pass1);
+	}
 	// cannot fail
-	if (subject) {
-		pass1 = uwsgi_regexp_apply_ovec(subject, subject_len, data, data_len, ur->ovector, ur->ovn);
+	else if (subject) {
+		pass1 = uwsgi_regexp_apply_ovec(subject, subject_len, data, data_len, ur->ovector[wsgi_req->async_id], ur->ovn[wsgi_req->async_id]);
 		pass1_len = strlen(pass1);
 	}
 
@@ -146,10 +150,25 @@ error:
 	return NULL;
 }
 
+static void uwsgi_routing_reset_memory(struct wsgi_request *wsgi_req, struct uwsgi_route *routes) {
+	// free dynamic memory structures
+	if (routes->if_func) {
+                        routes->ovn[wsgi_req->async_id] = 0;
+                        if (routes->ovector[wsgi_req->async_id]) {
+                                free(routes->ovector[wsgi_req->async_id]);
+                                routes->ovector[wsgi_req->async_id] = NULL;
+                        }
+                        if (routes->condition_ub[wsgi_req->async_id]) {
+                                uwsgi_buffer_destroy(routes->condition_ub[wsgi_req->async_id]);
+                                routes->condition_ub[wsgi_req->async_id] = NULL;
+                        }
+	}
+
+}
+
 int uwsgi_apply_routes_do(struct wsgi_request *wsgi_req, char *subject, uint16_t subject_len) {
 
 	struct uwsgi_route *routes = uwsgi.routes;
-        void *goon_func = NULL;
 	int n = -1;
 
 	char *orig_subject = subject;
@@ -163,10 +182,6 @@ int uwsgi_apply_routes_do(struct wsgi_request *wsgi_req, char *subject, uint16_t
 			goto next;
 		}
 
-		if (goon_func && goon_func == routes->func) {
-			goto next;
-		}
-		goon_func = NULL;
 		wsgi_req->route_goto = 0;
 
 		if (!routes->if_func) {
@@ -181,23 +196,26 @@ int uwsgi_apply_routes_do(struct wsgi_request *wsgi_req, char *subject, uint16_t
 				subject = *subject2 ;
 				subject_len = *subject_len2;
 			}
-			n = uwsgi_regexp_match_ovec(routes->pattern, routes->pattern_extra, subject, subject_len, routes->ovector, routes->ovn);
+			n = uwsgi_regexp_match_ovec(routes->pattern, routes->pattern_extra, subject, subject_len, routes->ovector[wsgi_req->async_id], routes->ovn[wsgi_req->async_id]);
 		}
 		else {
 			int ret = routes->if_func(wsgi_req, routes);
 			// error
 			if (ret < 0) {
+				uwsgi_routing_reset_memory(wsgi_req, routes);
 				return UWSGI_ROUTE_BREAK;
 			}
 			// true
 			if (!routes->if_negate) {
 				if (ret == 0) {
+					uwsgi_routing_reset_memory(wsgi_req, routes);
 					goto next;	
 				}
 				n = ret;
 			}
 			else {
 				if (ret > 0) {
+					uwsgi_routing_reset_memory(wsgi_req, routes);
 					goto next;	
 				}
 				n = 1;
@@ -208,6 +226,7 @@ run:
 		if (n >= 0) {
 			wsgi_req->is_routing = 1;
 			int ret = routes->func(wsgi_req, routes);
+			uwsgi_routing_reset_memory(wsgi_req, routes);
 			wsgi_req->is_routing = 0;
 			if (ret == UWSGI_ROUTE_BREAK) {
 				uwsgi.workers[uwsgi.mywid].cores[wsgi_req->async_id].routed_requests++;
@@ -217,10 +236,6 @@ run:
 				return ret;
 			}
 			
-			if (ret == UWSGI_ROUTE_GOON) {
-				goon_func = routes->func;
-			}
-
 			if (ret == -1) {
 				return UWSGI_ROUTE_BREAK;
 			}
@@ -295,14 +310,14 @@ void uwsgi_opt_add_route(char *opt, char *value, void *foobar) {
 		return;
 	}
 
-	char *route = uwsgi_str(value);
+	ur->orig_route = uwsgi_str(value);
 
 	if (!strcmp(foobar, "run")) {
-		command = route;	
+		command = ur->orig_route;	
 		goto done;
 	}
 
-	space = strchr(route, ' ');
+	space = strchr(ur->orig_route, ' ');
 	if (!space) {
 		uwsgi_log("invalid route syntax\n");
 		exit(1);
@@ -311,7 +326,7 @@ void uwsgi_opt_add_route(char *opt, char *value, void *foobar) {
 	*space = 0;
 
 	if (!strcmp(foobar, "if") || !strcmp(foobar, "if-not")) {
-		char *colon = strchr(route, ':');
+		char *colon = strchr(ur->orig_route, ':');
 		if (!colon) {
 			uwsgi_log("invalid route condition syntax\n");
                 	exit(1);
@@ -323,9 +338,9 @@ void uwsgi_opt_add_route(char *opt, char *value, void *foobar) {
 		}
 
 		foobar = colon+1;
-		ur->if_func = uwsgi_route_get_condition_func(route);
+		ur->if_func = uwsgi_route_get_condition_func(ur->orig_route);
 		if (!ur->if_func) {
-			uwsgi_log("unable to find \"%s\" route condition\n", route);
+			uwsgi_log("unable to find \"%s\" route condition\n", ur->orig_route);
 			exit(1);
 		}
 	}
@@ -365,18 +380,7 @@ void uwsgi_opt_add_route(char *opt, char *value, void *foobar) {
 
 	ur->subject_str = foobar;
 	ur->subject_str_len = strlen(ur->subject_str);
-	ur->regexp = route;
-
-	if (ur->subject && ur->subject_len) {
-		if (uwsgi_regexp_build(route, &ur->pattern, &ur->pattern_extra)) {
-			exit(1);
-		}
-
-		ur->ovn = uwsgi_regexp_ovector(ur->pattern, ur->pattern_extra);
-		if (ur->ovn > 0) {
-			ur->ovector = uwsgi_calloc(sizeof(int) * (3 * (ur->ovn + 1)));
-		}
-	}
+	ur->regexp = ur->orig_route;
 
 	command = space + 1;
 done:
@@ -394,16 +398,6 @@ done:
 	while (r) {
 		if (!strcmp(r->name, command)) {
 			if (r->func(ur, colon + 1) == 0) {
-				// apply is_last
-				struct uwsgi_route *last_ur = ur;
-				ur = uwsgi.routes;
-				while (ur) {
-					if (ur->func == last_ur->func) {
-						ur->is_last = 0;
-					}
-					ur = ur->next;
-				}
-				last_ur->is_last = 1;
 				return;
 			}
 			break;
@@ -415,6 +409,32 @@ done:
 	exit(1);
 }
 
+void uwsgi_fixup_routes() {
+	struct uwsgi_route *ur = uwsgi.routes;
+	while(ur) {
+		// prepare the main pointers
+		ur->ovn = uwsgi_calloc(sizeof(int) * uwsgi.cores);
+		ur->ovector = uwsgi_calloc(sizeof(int *) * uwsgi.cores);
+		ur->condition_ub = uwsgi_calloc( sizeof(struct uwsgi_buffer *) * uwsgi.cores);
+
+		// fill them if needed... (this is an optimization for route with a static subject)
+		if (ur->subject && ur->subject_len) {
+                	if (uwsgi_regexp_build(ur->orig_route, &ur->pattern, &ur->pattern_extra)) {
+                        	exit(1);
+                	}
+
+			int i;
+			for(i=0;i<uwsgi.cores;i++) {
+                		ur->ovn[i] = uwsgi_regexp_ovector(ur->pattern, ur->pattern_extra);
+                		if (ur->ovn[i] > 0) {
+                        		ur->ovector[i] = uwsgi_calloc(sizeof(int) * (3 * (ur->ovn[i] + 1)));
+                		}
+			}
+		}
+		ur = ur->next;
+        }
+}
+
 int uwsgi_route_api_func(struct wsgi_request *wsgi_req, char *router, char *args) {
 	struct uwsgi_route *ur = NULL;
 	struct uwsgi_router *r = uwsgi.routers;
@@ -476,30 +496,21 @@ static int uwsgi_router_break(struct uwsgi_route *ur, char *arg) {
 	return 0;
 }
 
-// goon route
-static int uwsgi_router_goon_func(struct wsgi_request *wsgi_req, struct uwsgi_route *route) {
-	return UWSGI_ROUTE_GOON;	
-}
-static int uwsgi_router_goon(struct uwsgi_route *ur, char *arg) {
-	ur->func = uwsgi_router_goon_func;
-	return 0;
-}
-
 // flush response
-static int transform_flush(struct wsgi_request *wsgi_req, struct uwsgi_buffer *ub, struct uwsgi_buffer **foobar, void *data) {
-	wsgi_req->flush = 1;
-	return uwsgi_response_write_body_do(wsgi_req, ub->buf, ub->pos);
-        return 0;
+static int transform_flush(struct wsgi_request *wsgi_req, struct uwsgi_transformation *ut) {
+	// avoid loops !!!
+	if (ut->chunk->pos == 0) return 0;
+	wsgi_req->transformed_chunk = ut->chunk->buf;
+	wsgi_req->transformed_chunk_len = ut->chunk->pos;
+	int ret = uwsgi_response_write_body_do(wsgi_req, ut->chunk->buf, ut->chunk->pos);
+	wsgi_req->transformed_chunk = NULL;
+	wsgi_req->transformed_chunk_len = 0;
+	ut->flushed = 1;
+	return ret;
 }
 static int uwsgi_router_flush_func(struct wsgi_request *wsgi_req, struct uwsgi_route *route) {
-	// if there are no transformations defined, set the flush right now
-	if (!wsgi_req->transformations) {
-		wsgi_req->flush = 1;
-	}
-	else {
-		// otherwise push it in the chain
-		uwsgi_add_transformation(wsgi_req, transform_flush, NULL);
-	}
+	struct uwsgi_transformation *ut = uwsgi_add_transformation(wsgi_req, transform_flush, NULL);
+	ut->can_stream = 1;
 	return UWSGI_ROUTE_NEXT;	
 }
 static int uwsgi_router_flush(struct uwsgi_route *ur, char *arg) {
@@ -507,6 +518,28 @@ static int uwsgi_router_flush(struct uwsgi_route *ur, char *arg) {
         return 0;
 }
 
+// fix content length
+static int transform_fixcl(struct wsgi_request *wsgi_req, struct uwsgi_transformation *ut) {
+	char buf[sizeof(UMAX64_STR)+1];
+        int ret = snprintf(buf, sizeof(UMAX64_STR)+1, "%llu", (unsigned long long) ut->chunk->pos);
+        if (ret <= 0 || ret > (int) (sizeof(UMAX64_STR)+1)) {
+                wsgi_req->write_errors++;
+                return -1;
+        }
+	// do not check for errors !!!
+        uwsgi_response_add_header(wsgi_req, "Content-Length", 14, buf, ret);
+	return 0;
+}
+static int uwsgi_router_fixcl_func(struct wsgi_request *wsgi_req, struct uwsgi_route *route) {
+        uwsgi_add_transformation(wsgi_req, transform_fixcl, NULL);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_fixcl(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_fixcl_func;
+        return 0;
+}
+
+
 // log route
 static int uwsgi_router_log_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
 
@@ -528,6 +561,16 @@ static int uwsgi_router_log(struct uwsgi_route *ur, char *arg) {
 	return 0;
 }
 
+// do not log !!!
+static int uwsgi_router_donotlog_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+	wsgi_req->do_not_log = 1;
+	return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_donotlog(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_donotlog_func;
+        return 0;
+}
+
 // logvar route
 static int uwsgi_router_logvar_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
 
@@ -733,6 +776,62 @@ static int uwsgi_router_setapp(struct uwsgi_route *ur, char *arg) {
         return 0;
 }
 
+// setscriptname route
+static int uwsgi_router_setscriptname_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+        if (!ub) return UWSGI_ROUTE_BREAK;
+        char *ptr = uwsgi_req_append(wsgi_req, "SCRIPT_NAME", 11, ub->buf, ub->pos);
+        if (!ptr) {
+                uwsgi_buffer_destroy(ub);
+                return UWSGI_ROUTE_BREAK;
+        }
+        wsgi_req->script_name = ptr;
+        wsgi_req->script_name_len = ub->pos;
+        uwsgi_buffer_destroy(ub);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_setscriptname(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_setscriptname_func;
+        ur->data = arg;
+        ur->data_len = strlen(arg);
+        return 0;
+}
+
+
+// setuser route
+static int uwsgi_router_setuser_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
+        char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
+        uint16_t *subject_len = (uint16_t *)  (((char *)(wsgi_req))+ur->subject_len);
+
+        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, *subject, *subject_len, ur->data, ur->data_len);
+        if (!ub) return UWSGI_ROUTE_BREAK;
+	uint16_t user_len = ub->pos;
+	// stop at the first colon (useful for various tricks)
+	char *colon = memchr(ub->buf, ':', ub->pos);
+	if (colon) {
+		user_len = colon - ub->buf;
+	}
+        char *ptr = uwsgi_req_append(wsgi_req, "REMOTE_USER", 11, ub->buf, user_len);
+        if (!ptr) {
+                uwsgi_buffer_destroy(ub);
+                return UWSGI_ROUTE_BREAK;
+        }
+        wsgi_req->remote_user = ptr;
+        wsgi_req->remote_user_len = ub->pos;
+        uwsgi_buffer_destroy(ub);
+        return UWSGI_ROUTE_NEXT;
+}
+static int uwsgi_router_setuser(struct uwsgi_route *ur, char *arg) {
+        ur->func = uwsgi_router_setuser_func;
+        ur->data = arg;
+        ur->data_len = strlen(arg);
+        return 0;
+}
+
+
 // sethome route
 static int uwsgi_router_sethome_func(struct wsgi_request *wsgi_req, struct uwsgi_route *ur) {
         char **subject = (char **) (((char *)(wsgi_req))+ur->subject);
@@ -881,21 +980,25 @@ static int uwsgi_route_condition_regexp(struct wsgi_request *wsgi_req, struct uw
         char *semicolon = memchr(ur->subject_str, ';', ur->subject_str_len);
         if (!semicolon) return 0;
 
-        struct uwsgi_buffer *ub = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, ur->subject_str, semicolon - ur->subject_str);
-        if (!ub) return -1;
+        ur->condition_ub[wsgi_req->async_id] = uwsgi_routing_translate(wsgi_req, ur, NULL, 0, ur->subject_str, semicolon - ur->subject_str);
+        if (!ur->condition_ub[wsgi_req->async_id]) return -1;
 
 	pcre *pattern;
 	pcre_extra *pattern_extra;
 	char *re = uwsgi_concat2n(semicolon+1, ur->subject_str_len - ((semicolon+1) - ur->subject_str), "", 0);
 	if (uwsgi_regexp_build(re, &pattern, &pattern_extra)) {
 		free(re);
-		uwsgi_buffer_destroy(ub);
 		return -1;
 	}
 	free(re);
 
-	if (uwsgi_regexp_match(pattern, pattern_extra, ub->buf, ub->pos) >= 0) {
-		uwsgi_buffer_destroy(ub);
+	// a condition has no initialized vectors, let's create them
+	ur->ovn[wsgi_req->async_id] = uwsgi_regexp_ovector(pattern, pattern_extra);
+        if (ur->ovn[wsgi_req->async_id] > 0) {
+        	ur->ovector[wsgi_req->async_id] = uwsgi_calloc(sizeof(int) * (3 * (ur->ovn[wsgi_req->async_id] + 1)));
+        }
+
+	if (uwsgi_regexp_match_ovec(pattern, pattern_extra, ur->condition_ub[wsgi_req->async_id]->buf, ur->condition_ub[wsgi_req->async_id]->pos, ur->ovector[wsgi_req->async_id], ur->ovn[wsgi_req->async_id] ) >= 0) {
 		pcre_free(pattern);
 #ifdef PCRE_STUDY_JIT_COMPILE
 		pcre_free_study(pattern_extra);
@@ -905,7 +1008,6 @@ static int uwsgi_route_condition_regexp(struct wsgi_request *wsgi_req, struct uw
 		return 1;
 	}
 
-        uwsgi_buffer_destroy(ub);
 	pcre_free(pattern);
 #ifdef PCRE_STUDY_JIT_COMPILE
 	pcre_free_study(pattern_extra);
@@ -1125,7 +1227,7 @@ static int uwsgi_route_condition_endswith(struct wsgi_request *wsgi_req, struct
                 return -1;
         }
 
-	if (ub2->pos < ub->pos) goto zero;
+	if (ub2->pos > ub->pos) goto zero;
         if(!uwsgi_strncmp(ub->buf + (ub->pos - ub2->pos), ub2->pos, ub2->buf, ub2->pos)) {
                 uwsgi_buffer_destroy(ub);
                 uwsgi_buffer_destroy(ub2);
@@ -1184,6 +1286,11 @@ static char *uwsgi_route_var_uwsgi(struct wsgi_request *wsgi_req, char *key, uin
 		ret = uwsgi_num2str(uwsgi.mypid);
 		*vallen = strlen(ret);
 	}
+	else if (!uwsgi_strncmp(key, keylen, "uuid", 4)) {
+		ret = uwsgi_malloc(37);
+		uwsgi_uuid(ret);
+		*vallen = 36;
+	}
 
 	return ret;
 }
@@ -1200,6 +1307,19 @@ static char *uwsgi_route_var_mime(struct wsgi_request *wsgi_req, char *key, uint
         return ret;
 }
 
+static char *uwsgi_route_var_httptime(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
+        // 30+1
+        char *ht = uwsgi_calloc(31);
+	size_t t = uwsgi_str_num(key, keylen);
+        int len = uwsgi_http_date(uwsgi_now() + t, ht);
+	if (len == 0) {
+		free(ht);
+		return NULL;
+	}
+	*vallen = len;
+        return ht;
+}
+
 
 static char *uwsgi_route_var_time(struct wsgi_request *wsgi_req, char *key, uint16_t keylen, uint16_t *vallen) {
         char *ret = NULL;
@@ -1280,8 +1400,8 @@ void uwsgi_register_embedded_routers() {
 	uwsgi_register_router("continue", uwsgi_router_continue);
         uwsgi_register_router("last", uwsgi_router_continue);
         uwsgi_register_router("break", uwsgi_router_break);
-        uwsgi_register_router("goon", uwsgi_router_goon);
         uwsgi_register_router("log", uwsgi_router_log);
+        uwsgi_register_router("donotlog", uwsgi_router_donotlog);
         uwsgi_register_router("logvar", uwsgi_router_logvar);
         uwsgi_register_router("goto", uwsgi_router_goto);
         uwsgi_register_router("addvar", uwsgi_router_addvar);
@@ -1293,12 +1413,15 @@ void uwsgi_register_embedded_routers() {
         uwsgi_register_router("send-crnl", uwsgi_router_send_crnl);
         uwsgi_register_router("chdir", uwsgi_router_chdir);
         uwsgi_register_router("setapp", uwsgi_router_setapp);
+        uwsgi_register_router("setuser", uwsgi_router_setuser);
         uwsgi_register_router("sethome", uwsgi_router_sethome);
         uwsgi_register_router("setfile", uwsgi_router_setfile);
+        uwsgi_register_router("setscriptname", uwsgi_router_setscriptname);
         uwsgi_register_router("setprocname", uwsgi_router_setprocname);
         uwsgi_register_router("alarm", uwsgi_router_alarm);
 
         uwsgi_register_router("flush", uwsgi_router_flush);
+        uwsgi_register_router("fixcl", uwsgi_router_fixcl);
 
         uwsgi_register_route_condition("exists", uwsgi_route_condition_exists);
         uwsgi_register_route_condition("isfile", uwsgi_route_condition_isfile);
@@ -1336,6 +1459,8 @@ void uwsgi_register_embedded_routers() {
 	urv->need_free = 1;
         urv = uwsgi_register_route_var("time", uwsgi_route_var_time);
 	urv->need_free = 1;
+        urv = uwsgi_register_route_var("httptime", uwsgi_route_var_httptime);
+	urv->need_free = 1;
 #ifdef UWSGI_MATHEVAL
         urv = uwsgi_register_route_var("math", uwsgi_route_var_math);
 	urv->need_free = 1;
