@@ -55,7 +55,7 @@ gzip:
 	return 1;
 }
 
-static int set_http_date(time_t t, char *dst) {
+int uwsgi_http_date(time_t t, char *dst) {
 
         static char *week[] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
         static char *months[] = {
@@ -179,7 +179,7 @@ int uwsgi_add_expires_type(struct wsgi_request *wsgi_req, char *mime_type, int m
 	while (udd) {
 		if (!uwsgi_strncmp(udd->key, udd->keylen, mime_type, mime_type_len)) {
 			int delta = uwsgi_str_num(udd->value, udd->vallen);
-			int size = set_http_date(now + delta, expires);
+			int size = uwsgi_http_date(now + delta, expires);
 			if (size > 0) {
 				if (uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, size)) return -1;
 			}
@@ -192,7 +192,7 @@ int uwsgi_add_expires_type(struct wsgi_request *wsgi_req, char *mime_type, int m
 	while (udd) {
 		if (!uwsgi_strncmp(udd->key, udd->keylen, mime_type, mime_type_len)) {
 			int delta = uwsgi_str_num(udd->value, udd->vallen);
-			int size = set_http_date(st->st_mtime + delta, expires);
+			int size = uwsgi_http_date(st->st_mtime + delta, expires);
 			if (size > 0) {
 				if (uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, size)) return -1;
 			}
@@ -215,7 +215,7 @@ int uwsgi_add_expires(struct wsgi_request *wsgi_req, char *filename, int filenam
 	while (udd) {
 		if (uwsgi_regexp_match(udd->pattern, udd->pattern_extra, filename, filename_len) >= 0) {
 			int delta = uwsgi_str_num(udd->value, udd->vallen);
-			int size = set_http_date(now + delta, expires);
+			int size = uwsgi_http_date(now + delta, expires);
 			if (size > 0) {
 				if (uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, size)) return -1;
 			}
@@ -228,7 +228,7 @@ int uwsgi_add_expires(struct wsgi_request *wsgi_req, char *filename, int filenam
 	while (udd) {
 		if (uwsgi_regexp_match(udd->pattern, udd->pattern_extra, filename, filename_len) >= 0) {
 			int delta = uwsgi_str_num(udd->value, udd->vallen);
-			int size = set_http_date(st->st_mtime + delta, expires);
+			int size = uwsgi_http_date(st->st_mtime + delta, expires);
 			if (size > 0) {
 				if (uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, size)) return -1;
 			}
@@ -250,7 +250,7 @@ int uwsgi_add_expires_path_info(struct wsgi_request *wsgi_req, struct stat *st)
 	while (udd) {
 		if (uwsgi_regexp_match(udd->pattern, udd->pattern_extra, wsgi_req->path_info, wsgi_req->path_info_len) >= 0) {
 			int delta = uwsgi_str_num(udd->value, udd->vallen);
-			int size = set_http_date(now + delta, expires);
+			int size = uwsgi_http_date(now + delta, expires);
 			if (size > 0) {
 				if (uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, size)) return -1;
 			}
@@ -263,7 +263,7 @@ int uwsgi_add_expires_path_info(struct wsgi_request *wsgi_req, struct stat *st)
 	while (udd) {
 		if (uwsgi_regexp_match(udd->pattern, udd->pattern_extra, wsgi_req->path_info, wsgi_req->path_info_len) >= 0) {
 			int delta = uwsgi_str_num(udd->value, udd->vallen);
-			int size = set_http_date(st->st_mtime + delta, expires);
+			int size = uwsgi_http_date(st->st_mtime + delta, expires);
 			if (size > 0) {
 				if (uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, size)) return -1;
 			}
@@ -285,7 +285,7 @@ int uwsgi_add_expires_uri(struct wsgi_request *wsgi_req, struct stat *st) {
 	while (udd) {
 		if (uwsgi_regexp_match(udd->pattern, udd->pattern_extra, wsgi_req->uri, wsgi_req->uri_len) >= 0) {
 			int delta = uwsgi_str_num(udd->value, udd->vallen);
-			int size = set_http_date(now + delta, expires);
+			int size = uwsgi_http_date(now + delta, expires);
 			if (size > 0) {
 				if (uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, size)) return -1;
 			}
@@ -298,7 +298,7 @@ int uwsgi_add_expires_uri(struct wsgi_request *wsgi_req, struct stat *st) {
 	while (udd) {
 		if (uwsgi_regexp_match(udd->pattern, udd->pattern_extra, wsgi_req->uri, wsgi_req->uri_len) >= 0) {
 			int delta = uwsgi_str_num(udd->value, udd->vallen);
-			int size = set_http_date(st->st_mtime + delta, expires);
+			int size = uwsgi_http_date(st->st_mtime + delta, expires);
 			if (size > 0) {
 				if (uwsgi_response_add_header(wsgi_req, "Expires", 7, expires, size)) return -1;
 			}
@@ -333,6 +333,10 @@ char *uwsgi_get_mime_type(char *name, int namelen, size_t *size) {
 	if (!ext)
 		return NULL;
 
+
+	if (uwsgi.threads > 1)
+                pthread_mutex_lock(&uwsgi.lock_static);
+
 	struct uwsgi_dyn_dict *udd = uwsgi.mimetypes;
 	while (udd) {
 		if (!uwsgi_strncmp(ext, count, udd->key, udd->keylen)) {
@@ -361,11 +365,16 @@ char *uwsgi_get_mime_type(char *name, int namelen, size_t *size) {
 				}
 			}
 			*size = udd->vallen;
+			if (uwsgi.threads > 1)
+                		pthread_mutex_unlock(&uwsgi.lock_static);
 			return udd->value;
 		}
 		udd = udd->next;
 	}
 
+	if (uwsgi.threads > 1)
+        	pthread_mutex_unlock(&uwsgi.lock_static);
+
 	return NULL;
 }
 
@@ -431,14 +440,12 @@ int uwsgi_real_file_serve(struct wsgi_request *wsgi_req, char *real_filename, si
 
 	size_t mime_type_size = 0;
 	char http_last_modified[49];
-
-	if (uwsgi.threads > 1)
-		pthread_mutex_lock(&uwsgi.lock_static);
+	int use_gzip = 0;
 
 	char *mime_type = uwsgi_get_mime_type(real_filename, real_filename_len, &mime_type_size);
 
-	if (uwsgi.threads > 1)
-		pthread_mutex_unlock(&uwsgi.lock_static);
+	// here we need to choose if we want the gzip variant;
+	if (uwsgi_static_want_gzip(wsgi_req, real_filename, real_filename_len, st)) use_gzip = 1;
 
 	if (wsgi_req->if_modified_since_len) {
 		time_t ims = parse_http_date(wsgi_req->if_modified_since, wsgi_req->if_modified_since_len);
@@ -451,6 +458,7 @@ int uwsgi_real_file_serve(struct wsgi_request *wsgi_req, char *real_filename, si
 	uwsgi_log("[uwsgi-fileserve] file %s found\n", real_filename);
 #endif
 
+
 	size_t fsize = st->st_size;
         if (wsgi_req->range_to) {
         	fsize = wsgi_req->range_to - wsgi_req->range_from;
@@ -462,7 +470,7 @@ int uwsgi_real_file_serve(struct wsgi_request *wsgi_req, char *real_filename, si
         	// reset in case of inconsistent size
         	if (wsgi_req->range_from > fsize) {
         		wsgi_req->range_from = 0;
-                	fsize = 0 ;
+        		fsize = 0;
         	}
 		else {
 			fsize -= wsgi_req->range_from;
@@ -483,6 +491,10 @@ int uwsgi_real_file_serve(struct wsgi_request *wsgi_req, char *real_filename, si
 	uwsgi_add_expires_uri(wsgi_req, st);
 #endif
 
+	if (use_gzip) {
+		if (uwsgi_response_add_header(wsgi_req, "Content-Encoding", 16, "gzip", 4)) return -1;
+	}
+
 	// Content-Type (if available)
 	if (mime_type_size > 0 && mime_type) {
 		if (uwsgi_response_add_content_type(wsgi_req, mime_type, mime_type_size)) return -1;
@@ -497,29 +509,25 @@ int uwsgi_real_file_serve(struct wsgi_request *wsgi_req, char *real_filename, si
 	if (uwsgi.file_serve_mode == 1) {
 		if (uwsgi_response_add_header(wsgi_req, "X-Accel-Redirect", 16, real_filename, real_filename_len)) return -1;
 		// this is the final header (\r\n added)
-		int size = set_http_date(st->st_mtime, http_last_modified);
+		int size = uwsgi_http_date(st->st_mtime, http_last_modified);
 		if (uwsgi_response_add_header(wsgi_req, "Last-Modified", 13, http_last_modified, size)) return -1;
 	}
 	// apache
 	else if (uwsgi.file_serve_mode == 2) {
 		if (uwsgi_response_add_header(wsgi_req, "X-Sendfile", 10, real_filename, real_filename_len)) return -1;
 		// this is the final header (\r\n added)
-		int size = set_http_date(st->st_mtime, http_last_modified);
+		int size = uwsgi_http_date(st->st_mtime, http_last_modified);
 		if (uwsgi_response_add_header(wsgi_req, "Last-Modified", 13, http_last_modified, size)) return -1;
 	}
 	// raw
 	else {
-		// here we need to choose if we want the gzip variant;
-		if (uwsgi_static_want_gzip(wsgi_req, real_filename, real_filename_len, st)) {
-			if (uwsgi_response_add_header(wsgi_req, "Content-Encoding", 16, "gzip", 4)) return -1;
-		}
 		// set Content-Length (to fsize NOT st->st_size)
 		if (uwsgi_response_add_content_length(wsgi_req, fsize)) return -1;
 		if (fsize > 0 && (wsgi_req->range_from || wsgi_req->range_to)) {
-			// here use teh original size !!!
+			// here use the original size !!!
 			if (uwsgi_response_add_content_range(wsgi_req, wsgi_req->range_from, wsgi_req->range_to, st->st_size)) return -1;
 		}
-		int size = set_http_date(st->st_mtime, http_last_modified);
+		int size = uwsgi_http_date(st->st_mtime, http_last_modified);
 		if (uwsgi_response_add_header(wsgi_req, "Last-Modified", 13, http_last_modified, size)) return -1;
 
 		// if it is a HEAD request just skip transfer
