@@ -335,28 +335,31 @@ shit:
         if (code_start) {
                 code_end = name_to_symbol_pkg(fullname2, "end");
                 if (code_end) {
+			char *symbolized;
                         PyObject *mod = PyImport_AddModule(fullname);
 			if (!mod) goto clear;
                         PyObject *dict = PyModule_GetDict(mod);
 			if (!dict) goto clear;
 
                         source = uwsgi_concat2n(code_start, code_end-code_start, "", 0);
-			modname = uwsgi_concat3("sym://", symbolize(fullname), "___init___py");
+			symbolized = symbolize(fullname);
+			modname = uwsgi_concat3("sym://", symbolized, "___init___py");
 
 			PyObject *pkgpath = Py_BuildValue("[O]", PyString_FromString(modname));
 
 			PyDict_SetItemString(dict, "__path__", pkgpath);
 			PyDict_SetItemString(dict, "__loader__", self);
 
-                        code = Py_CompileString(source, modname, Py_file_input);
-		if (!code) {
-			PyErr_Print();
-			goto shit2;
-		}
+			code = Py_CompileString(source, modname, Py_file_input);
+			if (!code) {
+				PyErr_Print();
+				goto shit2;
+			}
                         mod = PyImport_ExecCodeModuleEx(fullname, code, modname);
 
 			Py_DECREF(code);
 shit2:
+			free(symbolized);
 			free(source);
 			free(modname);
 			free(fullname2);
@@ -436,7 +439,7 @@ zipimporter_init(struct _symzipimporter *self, PyObject *args, PyObject *kwds)
 
         char *name;
         char *prefix = NULL;
-	int len = 0;
+	size_t len = 0;
 
         if (!PyArg_ParseTuple(args, "s", &name))
                 return -1;
@@ -466,6 +469,7 @@ zipimporter_init(struct _symzipimporter *self, PyObject *args, PyObject *kwds)
 
 	PyObject *stringio = PyImport_ImportModule("StringIO");
         if (!stringio) {
+		free(body);
                 return -1;
         }
 
@@ -572,18 +576,18 @@ symzipimporter_init(struct _symzipimporter *self, PyObject *args, PyObject *kwds
 	char *code_start = name_to_symbol(name, "start");
 	if (!code_start) {
 		PyErr_Format(PyExc_ValueError, "unable to find symbol");
-		return -1;
+		goto error;
 	}
 
 	char *code_end = name_to_symbol(name, "end");
 	if (!code_end) {
 		PyErr_Format(PyExc_ValueError, "unable to find symbol");
-		return -1;
+		goto error;
 	}
 
 	PyObject *stringio = PyImport_ImportModule("StringIO");
 	if (!stringio) {
-		return -1;
+		goto error;
 	}
 
 #ifdef PYTHREE
@@ -591,12 +595,12 @@ symzipimporter_init(struct _symzipimporter *self, PyObject *args, PyObject *kwds
 #else
 	PyObject *stringio_dict = PyModule_GetDict(stringio);
 	if (!stringio_dict) {
-		return -1;
+		goto error;
 	}
 
 	PyObject *stringio_stringio = PyDict_GetItemString(stringio_dict, "StringIO");
 	if (!stringio_stringio) {
-		return -1;
+		goto error;
 	}
 
 	PyObject *stringio_args = PyTuple_New(1);
@@ -606,12 +610,12 @@ symzipimporter_init(struct _symzipimporter *self, PyObject *args, PyObject *kwds
 	PyObject *source_code = PyInstance_New(stringio_stringio, stringio_args, NULL);
 #endif
 	if (!source_code) {
-		return -1;
+		goto error;
 	}
 
 	PyObject *zipfile = PyImport_ImportModule("zipfile");
 	if (!zipfile) {
-		return -1;
+		goto error;
 	}
 	
 #ifdef PYTHREE
@@ -619,12 +623,12 @@ symzipimporter_init(struct _symzipimporter *self, PyObject *args, PyObject *kwds
 #else
 	PyObject *zipfile_dict = PyModule_GetDict(zipfile);
         if (!zipfile_dict) {
-                return -1;
+		goto error;
         }
 
         PyObject *zipfile_zipfile = PyDict_GetItemString(zipfile_dict, "ZipFile");
         if (!zipfile_zipfile) {
-                return -1;
+		goto error;
         }
 
         PyObject *zipfile_args = PyTuple_New(1);
@@ -634,14 +638,14 @@ symzipimporter_init(struct _symzipimporter *self, PyObject *args, PyObject *kwds
 	self->zip = PyInstance_New(zipfile_zipfile, zipfile_args, NULL);
 #endif
         if (!self->zip) {
-                return -1;
+		goto error;
         }
 
 	Py_INCREF(self->zip);
 
 	self->items = PyObject_CallMethod(self->zip, "namelist", NULL);
         if (!self->items) {
-                return -1;
+		goto error;
         }
 
 	Py_INCREF(self->items);
@@ -654,6 +658,10 @@ symzipimporter_init(struct _symzipimporter *self, PyObject *args, PyObject *kwds
 
 
 	return 0;
+
+error:
+	free(name);
+	return -1;
 }
 
 static PyTypeObject SymZipImporter_Type = {
