@@ -463,7 +463,7 @@ VALUE rack_uwsgi_cache_get(int argc, VALUE *argv, VALUE *class) {
 	}
 
         uint64_t vallen = 0;;
-        char *value = uwsgi_cache_magic_get(key, keylen, &vallen, cache);
+        char *value = uwsgi_cache_magic_get(key, keylen, &vallen, NULL, cache);
 	if (value) {
         	VALUE res = rb_str_new(value, vallen);
 		free(value);
@@ -774,7 +774,7 @@ VALUE uwsgi_ruby_register_rpc(int argc, VALUE *argv, VALUE *class) {
         void *func = (void *) argv[1];
 
 
-        if (uwsgi_register_rpc(name, rack_plugin.modifier1, rb_argc, func)) {
+        if (uwsgi_register_rpc(name, &rack_plugin, rb_argc, func)) {
 clear:
                 rb_raise(rb_eRuntimeError, "unable to register rpc function");
                 return Qnil;
